#
# This file is part of pyasn1-modules software.
#
# Created by Stanisław Pitucha with asn1ate tool.
# Updated by Russ Housley for ORAddress Extension Attribute opentype support.
# Updated by Russ Housley for AlgorithmIdentifier opentype support.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Internet X.509 Public Key Infrastructure Certificate and Certificate
# Revocation List (CRL) Profile
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5280.txt
#
from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

MAX = float("inf")


def _buildOid(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


ub_e163_4_sub_address_length = univ.Integer(40)

ub_e163_4_number_length = univ.Integer(15)

unformatted_postal_address = univ.Integer(16)


class TerminalType(univ.Integer):
    pass


TerminalType.namedValues = namedval.NamedValues(
    ("telex", 3),
    ("teletex", 4),
    ("g3-facsimile", 5),
    ("g4-facsimile", 6),
    ("ia5-terminal", 7),
    ("videotex", 8),
)


class Extension(univ.Sequence):
    pass


Extension.componentType = namedtype.NamedTypes(
    namedtype.NamedType("extnID", univ.ObjectIdentifier()),
    namedtype.DefaultedNamedType("critical", univ.Boolean().subtype(value=0)),
    namedtype.NamedType("extnValue", univ.OctetString()),
)


class Extensions(univ.SequenceOf):
    pass


Extensions.componentType = Extension()
Extensions.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

physical_delivery_personal_name = univ.Integer(13)

ub_unformatted_address_length = univ.Integer(180)

ub_pds_parameter_length = univ.Integer(30)

ub_pds_physical_address_lines = univ.Integer(6)


class UnformattedPostalAddress(univ.Set):
    pass


UnformattedPostalAddress.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "printable-address",
        univ.SequenceOf(
            componentType=char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)
            )
        ),
    ),
    namedtype.OptionalNamedType(
        "teletex-string",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_unformatted_address_length)
        ),
    ),
)

ub_organization_name = univ.Integer(64)


class X520OrganizationName(univ.Choice):
    pass


X520OrganizationName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
        ),
    ),
)

ub_x121_address_length = univ.Integer(16)

pds_name = univ.Integer(7)

id_pkix = _buildOid(1, 3, 6, 1, 5, 5, 7)

id_kp = _buildOid(id_pkix, 3)

ub_postal_code_length = univ.Integer(16)


class PostalCode(univ.Choice):
    pass


PostalCode.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "numeric-code",
        char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length)
        ),
    ),
    namedtype.NamedType(
        "printable-code",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length)
        ),
    ),
)

ub_generation_qualifier_length = univ.Integer(3)

unique_postal_name = univ.Integer(20)


class DomainComponent(char.IA5String):
    pass


ub_domain_defined_attribute_value_length = univ.Integer(128)

ub_match = univ.Integer(128)

id_at = _buildOid(2, 5, 4)


class AttributeType(univ.ObjectIdentifier):
    pass


id_at_organizationalUnitName = _buildOid(id_at, 11)

terminal_type = univ.Integer(23)


class PDSParameter(univ.Set):
    pass


PDSParameter.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "printable-string",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)
        ),
    ),
    namedtype.OptionalNamedType(
        "teletex-string",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)
        ),
    ),
)


class PhysicalDeliveryPersonalName(PDSParameter):
    pass


ub_surname_length = univ.Integer(40)

id_ad = _buildOid(id_pkix, 48)

ub_domain_defined_attribute_type_length = univ.Integer(8)


class TeletexDomainDefinedAttribute(univ.Sequence):
    pass


TeletexDomainDefinedAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "type",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_domain_defined_attribute_type_length
            )
        ),
    ),
    namedtype.NamedType(
        "value",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_domain_defined_attribute_value_length
            )
        ),
    ),
)

ub_domain_defined_attributes = univ.Integer(4)


class TeletexDomainDefinedAttributes(univ.SequenceOf):
    pass


TeletexDomainDefinedAttributes.componentType = TeletexDomainDefinedAttribute()
TeletexDomainDefinedAttributes.sizeSpec = constraint.ValueSizeConstraint(
    1, ub_domain_defined_attributes
)

extended_network_address = univ.Integer(22)

ub_locality_name = univ.Integer(128)


class X520LocalityName(univ.Choice):
    pass


X520LocalityName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
        ),
    ),
)

teletex_organization_name = univ.Integer(3)

ub_given_name_length = univ.Integer(16)

ub_initials_length = univ.Integer(5)


class PersonalName(univ.Set):
    pass


PersonalName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "surname",
        char.PrintableString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)),
    ),
    namedtype.OptionalNamedType(
        "given-name",
        char.PrintableString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
    ),
    namedtype.OptionalNamedType(
        "initials",
        char.PrintableString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)),
    ),
    namedtype.OptionalNamedType(
        "generation-qualifier",
        char.PrintableString()
        .subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_generation_qualifier_length
            )
        )
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)),
    ),
)

ub_organizational_unit_name_length = univ.Integer(32)


class OrganizationalUnitName(char.PrintableString):
    pass


OrganizationalUnitName.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_organizational_unit_name_length
)

id_at_generationQualifier = _buildOid(id_at, 44)


class Version(univ.Integer):
    pass


Version.namedValues = namedval.NamedValues(("v1", 0), ("v2", 1), ("v3", 2))


class CertificateSerialNumber(univ.Integer):
    pass


algorithmIdentifierMap = {}


class AlgorithmIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algorithm", univ.ObjectIdentifier()),
        namedtype.OptionalNamedType(
            "parameters",
            univ.Any(),
            openType=opentype.OpenType("algorithm", algorithmIdentifierMap),
        ),
    )


class Time(univ.Choice):
    pass


Time.componentType = namedtype.NamedTypes(
    namedtype.NamedType("utcTime", useful.UTCTime()),
    namedtype.NamedType("generalTime", useful.GeneralizedTime()),
)


class AttributeValue(univ.Any):
    pass


certificateAttributesMap = {}


class AttributeTypeAndValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeType()),
        namedtype.NamedType(
            "value",
            AttributeValue(),
            openType=opentype.OpenType("type", certificateAttributesMap),
        ),
    )


class RelativeDistinguishedName(univ.SetOf):
    pass


RelativeDistinguishedName.componentType = AttributeTypeAndValue()
RelativeDistinguishedName.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class RDNSequence(univ.SequenceOf):
    pass


RDNSequence.componentType = RelativeDistinguishedName()


class Name(univ.Choice):
    pass


Name.componentType = namedtype.NamedTypes(
    namedtype.NamedType("rdnSequence", RDNSequence())
)


class TBSCertList(univ.Sequence):
    pass


TBSCertList.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("version", Version()),
    namedtype.NamedType("signature", AlgorithmIdentifier()),
    namedtype.NamedType("issuer", Name()),
    namedtype.NamedType("thisUpdate", Time()),
    namedtype.OptionalNamedType("nextUpdate", Time()),
    namedtype.OptionalNamedType(
        "revokedCertificates",
        univ.SequenceOf(
            componentType=univ.Sequence(
                componentType=namedtype.NamedTypes(
                    namedtype.NamedType("userCertificate", CertificateSerialNumber()),
                    namedtype.NamedType("revocationDate", Time()),
                    namedtype.OptionalNamedType("crlEntryExtensions", Extensions()),
                )
            )
        ),
    ),
    namedtype.OptionalNamedType(
        "crlExtensions",
        Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class CertificateList(univ.Sequence):
    pass


CertificateList.componentType = namedtype.NamedTypes(
    namedtype.NamedType("tbsCertList", TBSCertList()),
    namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
    namedtype.NamedType("signature", univ.BitString()),
)


class PhysicalDeliveryOfficeName(PDSParameter):
    pass


ub_extension_attributes = univ.Integer(256)

certificateExtensionsMap = {}

oraddressExtensionAttributeMap = {}


class ExtensionAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "extension-attribute-type",
            univ.Integer()
            .subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, ub_extension_attributes)
            )
            .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)),
        ),
        namedtype.NamedType(
            "extension-attribute-value",
            univ.Any().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
            openType=opentype.OpenType(
                "extension-attribute-type", oraddressExtensionAttributeMap
            ),
        ),
    )


id_qt = _buildOid(id_pkix, 2)

id_qt_cps = _buildOid(id_qt, 1)

id_at_stateOrProvinceName = _buildOid(id_at, 8)

id_at_title = _buildOid(id_at, 12)

id_at_serialNumber = _buildOid(id_at, 5)


class X520dnQualifier(char.PrintableString):
    pass


class PosteRestanteAddress(PDSParameter):
    pass


poste_restante_address = univ.Integer(19)


class UniqueIdentifier(univ.BitString):
    pass


class Validity(univ.Sequence):
    pass


Validity.componentType = namedtype.NamedTypes(
    namedtype.NamedType("notBefore", Time()), namedtype.NamedType("notAfter", Time())
)


class SubjectPublicKeyInfo(univ.Sequence):
    pass


SubjectPublicKeyInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("algorithm", AlgorithmIdentifier()),
    namedtype.NamedType("subjectPublicKey", univ.BitString()),
)


class TBSCertificate(univ.Sequence):
    pass


TBSCertificate.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        Version()
        .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v1"),
    ),
    namedtype.NamedType("serialNumber", CertificateSerialNumber()),
    namedtype.NamedType("signature", AlgorithmIdentifier()),
    namedtype.NamedType("issuer", Name()),
    namedtype.NamedType("validity", Validity()),
    namedtype.NamedType("subject", Name()),
    namedtype.NamedType("subjectPublicKeyInfo", SubjectPublicKeyInfo()),
    namedtype.OptionalNamedType(
        "issuerUniqueID",
        UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "subjectUniqueID",
        UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "extensions",
        Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
)

physical_delivery_office_name = univ.Integer(10)

ub_name = univ.Integer(32768)


class X520name(univ.Choice):
    pass


X520name.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
        ),
    ),
)

id_at_dnQualifier = _buildOid(id_at, 46)

ub_serial_number = univ.Integer(64)

ub_pseudonym = univ.Integer(128)

pkcs_9 = _buildOid(1, 2, 840, 113549, 1, 9)


class X121Address(char.NumericString):
    pass


X121Address.subtypeSpec = constraint.ValueSizeConstraint(1, ub_x121_address_length)


class NetworkAddress(X121Address):
    pass


ub_integer_options = univ.Integer(256)

id_at_commonName = _buildOid(id_at, 3)

ub_organization_name_length = univ.Integer(64)

id_ad_ocsp = _buildOid(id_ad, 1)

ub_country_name_numeric_length = univ.Integer(3)

ub_country_name_alpha_length = univ.Integer(2)


class PhysicalDeliveryCountryName(univ.Choice):
    pass


PhysicalDeliveryCountryName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "x121-dcc-code",
        char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                ub_country_name_numeric_length, ub_country_name_numeric_length
            )
        ),
    ),
    namedtype.NamedType(
        "iso-3166-alpha2-code",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                ub_country_name_alpha_length, ub_country_name_alpha_length
            )
        ),
    ),
)

id_emailAddress = _buildOid(pkcs_9, 1)

common_name = univ.Integer(1)


class X520Pseudonym(univ.Choice):
    pass


X520Pseudonym.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pseudonym)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pseudonym)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pseudonym)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pseudonym)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pseudonym)
        ),
    ),
)

ub_domain_name_length = univ.Integer(16)


class AdministrationDomainName(univ.Choice):
    pass


AdministrationDomainName.tagSet = univ.Choice.tagSet.tagExplicitly(
    tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 2)
)
AdministrationDomainName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "numeric",
        char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length)
        ),
    ),
    namedtype.NamedType(
        "printable",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length)
        ),
    ),
)


class PresentationAddress(univ.Sequence):
    pass


PresentationAddress.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "pSelector",
        univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "sSelector",
        univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "tSelector",
        univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "nAddresses",
        univ.SetOf(componentType=univ.OctetString()).subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
)


class ExtendedNetworkAddress(univ.Choice):
    pass


ExtendedNetworkAddress.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "e163-4-address",
        univ.Sequence(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "number",
                    char.NumericString()
                    .subtype(
                        subtypeSpec=constraint.ValueSizeConstraint(
                            1, ub_e163_4_number_length
                        )
                    )
                    .subtype(
                        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
                    ),
                ),
                namedtype.OptionalNamedType(
                    "sub-address",
                    char.NumericString()
                    .subtype(
                        subtypeSpec=constraint.ValueSizeConstraint(
                            1, ub_e163_4_sub_address_length
                        )
                    )
                    .subtype(
                        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
                    ),
                ),
            )
        ),
    ),
    namedtype.NamedType(
        "psap-address",
        PresentationAddress().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
)


class TeletexOrganizationName(char.TeletexString):
    pass


TeletexOrganizationName.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_organization_name_length
)

ub_terminal_id_length = univ.Integer(24)


class TerminalIdentifier(char.PrintableString):
    pass


TerminalIdentifier.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_terminal_id_length
)

id_ad_caIssuers = _buildOid(id_ad, 2)

id_at_countryName = _buildOid(id_at, 6)


class StreetAddress(PDSParameter):
    pass


postal_code = univ.Integer(9)

id_at_givenName = _buildOid(id_at, 42)

ub_title = univ.Integer(64)


class ExtensionAttributes(univ.SetOf):
    pass


ExtensionAttributes.componentType = ExtensionAttribute()
ExtensionAttributes.sizeSpec = constraint.ValueSizeConstraint(
    1, ub_extension_attributes
)

ub_emailaddress_length = univ.Integer(255)

id_ad_caRepository = _buildOid(id_ad, 5)


class ExtensionORAddressComponents(PDSParameter):
    pass


ub_organizational_unit_name = univ.Integer(64)


class X520OrganizationalUnitName(univ.Choice):
    pass


X520OrganizationalUnitName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)
        ),
    ),
)


class LocalPostalAttributes(PDSParameter):
    pass


teletex_organizational_unit_names = univ.Integer(5)


class X520Title(univ.Choice):
    pass


X520Title.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
        ),
    ),
)

id_at_localityName = _buildOid(id_at, 7)

id_at_initials = _buildOid(id_at, 43)

ub_state_name = univ.Integer(128)


class X520StateOrProvinceName(univ.Choice):
    pass


X520StateOrProvinceName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
        ),
    ),
)

physical_delivery_organization_name = univ.Integer(14)

id_at_surname = _buildOid(id_at, 4)


class X520countryName(char.PrintableString):
    pass


X520countryName.subtypeSpec = constraint.ValueSizeConstraint(2, 2)

physical_delivery_office_number = univ.Integer(11)

id_qt_unotice = _buildOid(id_qt, 2)


class X520SerialNumber(char.PrintableString):
    pass


X520SerialNumber.subtypeSpec = constraint.ValueSizeConstraint(1, ub_serial_number)


class Attribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeType()),
        namedtype.NamedType(
            "values",
            univ.SetOf(componentType=AttributeValue()),
            openType=opentype.OpenType("type", certificateAttributesMap),
        ),
    )


ub_common_name = univ.Integer(64)

id_pe = _buildOid(id_pkix, 1)


class ExtensionPhysicalDeliveryAddressComponents(PDSParameter):
    pass


class EmailAddress(char.IA5String):
    pass


EmailAddress.subtypeSpec = constraint.ValueSizeConstraint(1, ub_emailaddress_length)

id_at_organizationName = _buildOid(id_at, 10)

post_office_box_address = univ.Integer(18)


class BuiltInDomainDefinedAttribute(univ.Sequence):
    pass


BuiltInDomainDefinedAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "type",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_domain_defined_attribute_type_length
            )
        ),
    ),
    namedtype.NamedType(
        "value",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_domain_defined_attribute_value_length
            )
        ),
    ),
)


class BuiltInDomainDefinedAttributes(univ.SequenceOf):
    pass


BuiltInDomainDefinedAttributes.componentType = BuiltInDomainDefinedAttribute()
BuiltInDomainDefinedAttributes.sizeSpec = constraint.ValueSizeConstraint(
    1, ub_domain_defined_attributes
)

id_at_pseudonym = _buildOid(id_at, 65)

id_domainComponent = _buildOid(0, 9, 2342, 19200300, 100, 1, 25)


class X520CommonName(univ.Choice):
    pass


X520CommonName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
        ),
    ),
)

extension_OR_address_components = univ.Integer(12)

ub_organizational_units = univ.Integer(4)

teletex_personal_name = univ.Integer(4)

ub_numeric_user_id_length = univ.Integer(32)

ub_common_name_length = univ.Integer(64)


class TeletexCommonName(char.TeletexString):
    pass


TeletexCommonName.subtypeSpec = constraint.ValueSizeConstraint(1, ub_common_name_length)


class PhysicalDeliveryOrganizationName(PDSParameter):
    pass


extension_physical_delivery_address_components = univ.Integer(15)


class NumericUserIdentifier(char.NumericString):
    pass


NumericUserIdentifier.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_numeric_user_id_length
)


class CountryName(univ.Choice):
    pass


CountryName.tagSet = univ.Choice.tagSet.tagExplicitly(
    tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 1)
)
CountryName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "x121-dcc-code",
        char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                ub_country_name_numeric_length, ub_country_name_numeric_length
            )
        ),
    ),
    namedtype.NamedType(
        "iso-3166-alpha2-code",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                ub_country_name_alpha_length, ub_country_name_alpha_length
            )
        ),
    ),
)


class OrganizationName(char.PrintableString):
    pass


OrganizationName.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_organization_name_length
)


class OrganizationalUnitNames(univ.SequenceOf):
    pass


OrganizationalUnitNames.componentType = OrganizationalUnitName()
OrganizationalUnitNames.sizeSpec = constraint.ValueSizeConstraint(
    1, ub_organizational_units
)


class PrivateDomainName(univ.Choice):
    pass


PrivateDomainName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "numeric",
        char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length)
        ),
    ),
    namedtype.NamedType(
        "printable",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length)
        ),
    ),
)


class BuiltInStandardAttributes(univ.Sequence):
    pass


BuiltInStandardAttributes.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("country-name", CountryName()),
    namedtype.OptionalNamedType(
        "administration-domain-name", AdministrationDomainName()
    ),
    namedtype.OptionalNamedType(
        "network-address",
        NetworkAddress().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "terminal-identifier",
        TerminalIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "private-domain-name",
        PrivateDomainName().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "organization-name",
        OrganizationName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.OptionalNamedType(
        "numeric-user-identifier",
        NumericUserIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
    namedtype.OptionalNamedType(
        "personal-name",
        PersonalName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
        ),
    ),
    namedtype.OptionalNamedType(
        "organizational-unit-names",
        OrganizationalUnitNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)
        ),
    ),
)


class ORAddress(univ.Sequence):
    pass


ORAddress.componentType = namedtype.NamedTypes(
    namedtype.NamedType("built-in-standard-attributes", BuiltInStandardAttributes()),
    namedtype.OptionalNamedType(
        "built-in-domain-defined-attributes", BuiltInDomainDefinedAttributes()
    ),
    namedtype.OptionalNamedType("extension-attributes", ExtensionAttributes()),
)


class DistinguishedName(RDNSequence):
    pass


id_ad_timeStamping = _buildOid(id_ad, 3)


class PhysicalDeliveryOfficeNumber(PDSParameter):
    pass


teletex_domain_defined_attributes = univ.Integer(6)


class UniquePostalName(PDSParameter):
    pass


physical_delivery_country_name = univ.Integer(8)

ub_pds_name_length = univ.Integer(16)


class PDSName(char.PrintableString):
    pass


PDSName.subtypeSpec = constraint.ValueSizeConstraint(1, ub_pds_name_length)


class TeletexPersonalName(univ.Set):
    pass


TeletexPersonalName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "surname",
        char.TeletexString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)),
    ),
    namedtype.OptionalNamedType(
        "given-name",
        char.TeletexString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
    ),
    namedtype.OptionalNamedType(
        "initials",
        char.TeletexString()
        .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)),
    ),
    namedtype.OptionalNamedType(
        "generation-qualifier",
        char.TeletexString()
        .subtype(
            subtypeSpec=constraint.ValueSizeConstraint(
                1, ub_generation_qualifier_length
            )
        )
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)),
    ),
)

street_address = univ.Integer(17)


class PostOfficeBoxAddress(PDSParameter):
    pass


local_postal_attributes = univ.Integer(21)


class DirectoryString(univ.Choice):
    pass


DirectoryString.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX)),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX)),
    ),
)

teletex_common_name = univ.Integer(2)


class CommonName(char.PrintableString):
    pass


CommonName.subtypeSpec = constraint.ValueSizeConstraint(1, ub_common_name_length)


class Certificate(univ.Sequence):
    pass


Certificate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("tbsCertificate", TBSCertificate()),
    namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
    namedtype.NamedType("signature", univ.BitString()),
)


class TeletexOrganizationalUnitName(char.TeletexString):
    pass


TeletexOrganizationalUnitName.subtypeSpec = constraint.ValueSizeConstraint(
    1, ub_organizational_unit_name_length
)

id_at_name = _buildOid(id_at, 41)


class TeletexOrganizationalUnitNames(univ.SequenceOf):
    pass


TeletexOrganizationalUnitNames.componentType = TeletexOrganizationalUnitName()
TeletexOrganizationalUnitNames.sizeSpec = constraint.ValueSizeConstraint(
    1, ub_organizational_units
)

id_ce = _buildOid(2, 5, 29)

id_ce_issuerAltName = _buildOid(id_ce, 18)


class SkipCerts(univ.Integer):
    pass


SkipCerts.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class CRLReason(univ.Enumerated):
    pass


CRLReason.namedValues = namedval.NamedValues(
    ("unspecified", 0),
    ("keyCompromise", 1),
    ("cACompromise", 2),
    ("affiliationChanged", 3),
    ("superseded", 4),
    ("cessationOfOperation", 5),
    ("certificateHold", 6),
    ("removeFromCRL", 8),
    ("privilegeWithdrawn", 9),
    ("aACompromise", 10),
)


class PrivateKeyUsagePeriod(univ.Sequence):
    pass


PrivateKeyUsagePeriod.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "notBefore",
        useful.GeneralizedTime().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "notAfter",
        useful.GeneralizedTime().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


anotherNameMap = {}


class AnotherName(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type-id", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "value",
            univ.Any().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
            openType=opentype.OpenType("type-id", anotherNameMap),
        ),
    )


class EDIPartyName(univ.Sequence):
    pass


EDIPartyName.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "nameAssigner",
        DirectoryString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "partyName",
        DirectoryString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class GeneralName(univ.Choice):
    pass


GeneralName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "otherName",
        AnotherName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "rfc822Name",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "dNSName",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "x400Address",
        ORAddress().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.NamedType(
        "directoryName",
        Name().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
        ),
    ),
    namedtype.NamedType(
        "ediPartyName",
        EDIPartyName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
        ),
    ),
    namedtype.NamedType(
        "uniformResourceIdentifier",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)
        ),
    ),
    namedtype.NamedType(
        "iPAddress",
        univ.OctetString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
        ),
    ),
    namedtype.NamedType(
        "registeredID",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 8)
        ),
    ),
)


class BaseDistance(univ.Integer):
    pass


BaseDistance.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class GeneralSubtree(univ.Sequence):
    pass


GeneralSubtree.componentType = namedtype.NamedTypes(
    namedtype.NamedType("base", GeneralName()),
    namedtype.DefaultedNamedType(
        "minimum",
        BaseDistance()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value=0),
    ),
    namedtype.OptionalNamedType(
        "maximum",
        BaseDistance().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class GeneralNames(univ.SequenceOf):
    pass


GeneralNames.componentType = GeneralName()
GeneralNames.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class DistributionPointName(univ.Choice):
    pass


DistributionPointName.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "fullName",
        GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "nameRelativeToCRLIssuer",
        RelativeDistinguishedName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class ReasonFlags(univ.BitString):
    pass


ReasonFlags.namedValues = namedval.NamedValues(
    ("unused", 0),
    ("keyCompromise", 1),
    ("cACompromise", 2),
    ("affiliationChanged", 3),
    ("superseded", 4),
    ("cessationOfOperation", 5),
    ("certificateHold", 6),
    ("privilegeWithdrawn", 7),
    ("aACompromise", 8),
)


class IssuingDistributionPoint(univ.Sequence):
    pass


IssuingDistributionPoint.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "distributionPoint",
        DistributionPointName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.DefaultedNamedType(
        "onlyContainsUserCerts",
        univ.Boolean()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value=0),
    ),
    namedtype.DefaultedNamedType(
        "onlyContainsCACerts",
        univ.Boolean()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))
        .subtype(value=0),
    ),
    namedtype.OptionalNamedType(
        "onlySomeReasons",
        ReasonFlags().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.DefaultedNamedType(
        "indirectCRL",
        univ.Boolean()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4))
        .subtype(value=0),
    ),
    namedtype.DefaultedNamedType(
        "onlyContainsAttributeCerts",
        univ.Boolean()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5))
        .subtype(value=0),
    ),
)

id_ce_certificatePolicies = _buildOid(id_ce, 32)

id_kp_emailProtection = _buildOid(id_kp, 4)


class AccessDescription(univ.Sequence):
    pass


AccessDescription.componentType = namedtype.NamedTypes(
    namedtype.NamedType("accessMethod", univ.ObjectIdentifier()),
    namedtype.NamedType("accessLocation", GeneralName()),
)


class IssuerAltName(GeneralNames):
    pass


id_ce_cRLDistributionPoints = _buildOid(id_ce, 31)

holdInstruction = _buildOid(2, 2, 840, 10040, 2)

id_holdinstruction_callissuer = _buildOid(holdInstruction, 2)

id_ce_subjectDirectoryAttributes = _buildOid(id_ce, 9)

id_ce_issuingDistributionPoint = _buildOid(id_ce, 28)


class DistributionPoint(univ.Sequence):
    pass


DistributionPoint.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "distributionPoint",
        DistributionPointName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "reasons",
        ReasonFlags().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "cRLIssuer",
        GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class CRLDistributionPoints(univ.SequenceOf):
    pass


CRLDistributionPoints.componentType = DistributionPoint()
CRLDistributionPoints.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class GeneralSubtrees(univ.SequenceOf):
    pass


GeneralSubtrees.componentType = GeneralSubtree()
GeneralSubtrees.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class NameConstraints(univ.Sequence):
    pass


NameConstraints.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "permittedSubtrees",
        GeneralSubtrees().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "excludedSubtrees",
        GeneralSubtrees().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class SubjectDirectoryAttributes(univ.SequenceOf):
    pass


SubjectDirectoryAttributes.componentType = Attribute()
SubjectDirectoryAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_kp_OCSPSigning = _buildOid(id_kp, 9)

id_kp_timeStamping = _buildOid(id_kp, 8)


class DisplayText(univ.Choice):
    pass


DisplayText.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "ia5String",
        char.IA5String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200)),
    ),
    namedtype.NamedType(
        "visibleString",
        char.VisibleString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 200)
        ),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200)),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200)),
    ),
)


class NoticeReference(univ.Sequence):
    pass


NoticeReference.componentType = namedtype.NamedTypes(
    namedtype.NamedType("organization", DisplayText()),
    namedtype.NamedType("noticeNumbers", univ.SequenceOf(componentType=univ.Integer())),
)


class UserNotice(univ.Sequence):
    pass


UserNotice.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("noticeRef", NoticeReference()),
    namedtype.OptionalNamedType("explicitText", DisplayText()),
)


class PolicyQualifierId(univ.ObjectIdentifier):
    pass


policyQualifierInfoMap = {}


class PolicyQualifierInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("policyQualifierId", PolicyQualifierId()),
        namedtype.NamedType(
            "qualifier",
            univ.Any(),
            openType=opentype.OpenType("policyQualifierId", policyQualifierInfoMap),
        ),
    )


class CertPolicyId(univ.ObjectIdentifier):
    pass


class PolicyInformation(univ.Sequence):
    pass


PolicyInformation.componentType = namedtype.NamedTypes(
    namedtype.NamedType("policyIdentifier", CertPolicyId()),
    namedtype.OptionalNamedType(
        "policyQualifiers", univ.SequenceOf(componentType=PolicyQualifierInfo())
    ),
)


class CertificatePolicies(univ.SequenceOf):
    pass


CertificatePolicies.componentType = PolicyInformation()
CertificatePolicies.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class SubjectAltName(GeneralNames):
    pass


id_ce_basicConstraints = _buildOid(id_ce, 19)

id_ce_authorityKeyIdentifier = _buildOid(id_ce, 35)

id_kp_codeSigning = _buildOid(id_kp, 3)


class BasicConstraints(univ.Sequence):
    pass


BasicConstraints.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("cA", univ.Boolean().subtype(value=0)),
    namedtype.OptionalNamedType(
        "pathLenConstraint",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
)

id_ce_certificateIssuer = _buildOid(id_ce, 29)


class PolicyMappings(univ.SequenceOf):
    pass


PolicyMappings.componentType = univ.Sequence(
    componentType=namedtype.NamedTypes(
        namedtype.NamedType("issuerDomainPolicy", CertPolicyId()),
        namedtype.NamedType("subjectDomainPolicy", CertPolicyId()),
    )
)

PolicyMappings.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class InhibitAnyPolicy(SkipCerts):
    pass


anyPolicy = _buildOid(id_ce_certificatePolicies, 0)


class CRLNumber(univ.Integer):
    pass


CRLNumber.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class BaseCRLNumber(CRLNumber):
    pass


id_ce_nameConstraints = _buildOid(id_ce, 30)

id_kp_serverAuth = _buildOid(id_kp, 1)

id_ce_freshestCRL = _buildOid(id_ce, 46)

id_ce_cRLReasons = _buildOid(id_ce, 21)

id_ce_extKeyUsage = _buildOid(id_ce, 37)


class KeyIdentifier(univ.OctetString):
    pass


class AuthorityKeyIdentifier(univ.Sequence):
    pass


AuthorityKeyIdentifier.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "keyIdentifier",
        KeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "authorityCertIssuer",
        GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "authorityCertSerialNumber",
        CertificateSerialNumber().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class FreshestCRL(CRLDistributionPoints):
    pass


id_ce_policyConstraints = _buildOid(id_ce, 36)

id_pe_authorityInfoAccess = _buildOid(id_pe, 1)


class AuthorityInfoAccessSyntax(univ.SequenceOf):
    pass


AuthorityInfoAccessSyntax.componentType = AccessDescription()
AuthorityInfoAccessSyntax.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_holdinstruction_none = _buildOid(holdInstruction, 1)


class CPSuri(char.IA5String):
    pass


id_pe_subjectInfoAccess = _buildOid(id_pe, 11)


class SubjectKeyIdentifier(KeyIdentifier):
    pass


id_ce_subjectAltName = _buildOid(id_ce, 17)


class KeyPurposeId(univ.ObjectIdentifier):
    pass


class ExtKeyUsageSyntax(univ.SequenceOf):
    pass


ExtKeyUsageSyntax.componentType = KeyPurposeId()
ExtKeyUsageSyntax.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class HoldInstructionCode(univ.ObjectIdentifier):
    pass


id_ce_deltaCRLIndicator = _buildOid(id_ce, 27)

id_ce_keyUsage = _buildOid(id_ce, 15)

id_ce_holdInstructionCode = _buildOid(id_ce, 23)


class SubjectInfoAccessSyntax(univ.SequenceOf):
    pass


SubjectInfoAccessSyntax.componentType = AccessDescription()
SubjectInfoAccessSyntax.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class InvalidityDate(useful.GeneralizedTime):
    pass


class KeyUsage(univ.BitString):
    pass


KeyUsage.namedValues = namedval.NamedValues(
    ("digitalSignature", 0),
    ("nonRepudiation", 1),
    ("keyEncipherment", 2),
    ("dataEncipherment", 3),
    ("keyAgreement", 4),
    ("keyCertSign", 5),
    ("cRLSign", 6),
    ("encipherOnly", 7),
    ("decipherOnly", 8),
)

id_ce_invalidityDate = _buildOid(id_ce, 24)

id_ce_policyMappings = _buildOid(id_ce, 33)

anyExtendedKeyUsage = _buildOid(id_ce_extKeyUsage, 0)

id_ce_privateKeyUsagePeriod = _buildOid(id_ce, 16)

id_ce_cRLNumber = _buildOid(id_ce, 20)


class CertificateIssuer(GeneralNames):
    pass


id_holdinstruction_reject = _buildOid(holdInstruction, 3)


class PolicyConstraints(univ.Sequence):
    pass


PolicyConstraints.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "requireExplicitPolicy",
        SkipCerts().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "inhibitPolicyMapping",
        SkipCerts().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)

id_kp_clientAuth = _buildOid(id_kp, 2)

id_ce_subjectKeyIdentifier = _buildOid(id_ce, 14)

id_ce_inhibitAnyPolicy = _buildOid(id_ce, 54)

# map of ORAddress ExtensionAttribute type to ExtensionAttribute value

_oraddressExtensionAttributeMapUpdate = {
    common_name: CommonName(),
    teletex_common_name: TeletexCommonName(),
    teletex_organization_name: TeletexOrganizationName(),
    teletex_personal_name: TeletexPersonalName(),
    teletex_organizational_unit_names: TeletexOrganizationalUnitNames(),
    pds_name: PDSName(),
    physical_delivery_country_name: PhysicalDeliveryCountryName(),
    postal_code: PostalCode(),
    physical_delivery_office_name: PhysicalDeliveryOfficeName(),
    physical_delivery_office_number: PhysicalDeliveryOfficeNumber(),
    extension_OR_address_components: ExtensionORAddressComponents(),
    physical_delivery_personal_name: PhysicalDeliveryPersonalName(),
    physical_delivery_organization_name: PhysicalDeliveryOrganizationName(),
    extension_physical_delivery_address_components: ExtensionPhysicalDeliveryAddressComponents(),
    unformatted_postal_address: UnformattedPostalAddress(),
    street_address: StreetAddress(),
    post_office_box_address: PostOfficeBoxAddress(),
    poste_restante_address: PosteRestanteAddress(),
    unique_postal_name: UniquePostalName(),
    local_postal_attributes: LocalPostalAttributes(),
    extended_network_address: ExtendedNetworkAddress(),
    terminal_type: TerminalType(),
    teletex_domain_defined_attributes: TeletexDomainDefinedAttributes(),
}

oraddressExtensionAttributeMap.update(_oraddressExtensionAttributeMapUpdate)


# map of AttributeType -> AttributeValue

_certificateAttributesMapUpdate = {
    id_at_name: X520name(),
    id_at_surname: X520name(),
    id_at_givenName: X520name(),
    id_at_initials: X520name(),
    id_at_generationQualifier: X520name(),
    id_at_commonName: X520CommonName(),
    id_at_localityName: X520LocalityName(),
    id_at_stateOrProvinceName: X520StateOrProvinceName(),
    id_at_organizationName: X520OrganizationName(),
    id_at_organizationalUnitName: X520OrganizationalUnitName(),
    id_at_title: X520Title(),
    id_at_dnQualifier: X520dnQualifier(),
    id_at_countryName: X520countryName(),
    id_at_serialNumber: X520SerialNumber(),
    id_at_pseudonym: X520Pseudonym(),
    id_domainComponent: DomainComponent(),
    id_emailAddress: EmailAddress(),
}

certificateAttributesMap.update(_certificateAttributesMapUpdate)


# map of Certificate Extension OIDs to Extensions

_certificateExtensionsMap = {
    id_ce_authorityKeyIdentifier: AuthorityKeyIdentifier(),
    id_ce_subjectKeyIdentifier: SubjectKeyIdentifier(),
    id_ce_keyUsage: KeyUsage(),
    id_ce_privateKeyUsagePeriod: PrivateKeyUsagePeriod(),
    id_ce_certificatePolicies: CertificatePolicies(),
    id_ce_policyMappings: PolicyMappings(),
    id_ce_subjectAltName: SubjectAltName(),
    id_ce_issuerAltName: IssuerAltName(),
    id_ce_subjectDirectoryAttributes: SubjectDirectoryAttributes(),
    id_ce_basicConstraints: BasicConstraints(),
    id_ce_nameConstraints: NameConstraints(),
    id_ce_policyConstraints: PolicyConstraints(),
    id_ce_extKeyUsage: ExtKeyUsageSyntax(),
    id_ce_cRLDistributionPoints: CRLDistributionPoints(),
    id_pe_authorityInfoAccess: AuthorityInfoAccessSyntax(),
    id_ce_cRLNumber: univ.Integer(),
    id_ce_deltaCRLIndicator: BaseCRLNumber(),
    id_ce_issuingDistributionPoint: IssuingDistributionPoint(),
    id_ce_cRLReasons: CRLReason(),
    id_ce_holdInstructionCode: univ.ObjectIdentifier(),
    id_ce_invalidityDate: useful.GeneralizedTime(),
    id_ce_certificateIssuer: GeneralNames(),
}

certificateExtensionsMap.update(_certificateExtensionsMap)



# This file is being contributed to of pyasn1-modules software.
#
# Created by Russ Housley without assistance from the asn1ate tool.
# Modified by Russ Housley to add support for opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CMS Key Package Receipt and Error Content Types
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7191.txt

from pyasn1.type import constraint, namedtype, namedval, opentype, tag, univ

from pyasn1_modules import rfc5280, rfc5652

MAX = float("inf")

DistinguishedName = rfc5280.DistinguishedName


# SingleAttribute is the same as Attribute in RFC 5652, except that the
# attrValues SET must have one and only one member


class AttributeValue(univ.Any):
    pass


class AttributeValues(univ.SetOf):
    pass


AttributeValues.componentType = AttributeValue()
AttributeValues.sizeSpec = univ.Set.sizeSpec + constraint.ValueSizeConstraint(1, 1)


class SingleAttribute(univ.Sequence):
    pass


SingleAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "attrValues",
        AttributeValues(),
        openType=opentype.OpenType("attrType", rfc5652.cmsAttributesMap),
    ),
)


# SIR Entity Name


class SIREntityNameType(univ.ObjectIdentifier):
    pass


class SIREntityNameValue(univ.Any):
    pass


class SIREntityName(univ.Sequence):
    pass


SIREntityName.componentType = namedtype.NamedTypes(
    namedtype.NamedType("sirenType", SIREntityNameType()),
    namedtype.NamedType("sirenValue", univ.OctetString())
    # CONTAINING the DER-encoded SIREntityNameValue
)


class SIREntityNames(univ.SequenceOf):
    pass


SIREntityNames.componentType = SIREntityName()
SIREntityNames.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


id_dn = univ.ObjectIdentifier("2.16.840.1.101.2.1.16.0")


class siren_dn(SIREntityName):
    def __init__(self):
        SIREntityName.__init__(self)
        self["sirenType"] = id_dn


# Key Package Error CMS Content Type


class EnumeratedErrorCode(univ.Enumerated):
    pass


# Error codes with values <= 33 are aligned with RFC 5934
EnumeratedErrorCode.namedValues = namedval.NamedValues(
    ("decodeFailure", 1),
    ("badContentInfo", 2),
    ("badSignedData", 3),
    ("badEncapContent", 4),
    ("badCertificate", 5),
    ("badSignerInfo", 6),
    ("badSignedAttrs", 7),
    ("badUnsignedAttrs", 8),
    ("missingContent", 9),
    ("noTrustAnchor", 10),
    ("notAuthorized", 11),
    ("badDigestAlgorithm", 12),
    ("badSignatureAlgorithm", 13),
    ("unsupportedKeySize", 14),
    ("unsupportedParameters", 15),
    ("signatureFailure", 16),
    ("insufficientMemory", 17),
    ("incorrectTarget", 23),
    ("missingSignature", 29),
    ("resourcesBusy", 30),
    ("versionNumberMismatch", 31),
    ("revokedCertificate", 33),
    ("ambiguousDecrypt", 60),
    ("noDecryptKey", 61),
    ("badEncryptedData", 62),
    ("badEnvelopedData", 63),
    ("badAuthenticatedData", 64),
    ("badAuthEnvelopedData", 65),
    ("badKeyAgreeRecipientInfo", 66),
    ("badKEKRecipientInfo", 67),
    ("badEncryptContent", 68),
    ("badEncryptAlgorithm", 69),
    ("missingCiphertext", 70),
    ("decryptFailure", 71),
    ("badMACAlgorithm", 72),
    ("badAuthAttrs", 73),
    ("badUnauthAttrs", 74),
    ("invalidMAC", 75),
    ("mismatchedDigestAlg", 76),
    ("missingCertificate", 77),
    ("tooManySigners", 78),
    ("missingSignedAttributes", 79),
    ("derEncodingNotUsed", 80),
    ("missingContentHints", 81),
    ("invalidAttributeLocation", 82),
    ("badMessageDigest", 83),
    ("badKeyPackage", 84),
    ("badAttributes", 85),
    ("attributeComparisonFailure", 86),
    ("unsupportedSymmetricKeyPackage", 87),
    ("unsupportedAsymmetricKeyPackage", 88),
    ("constraintViolation", 89),
    ("ambiguousDefaultValue", 90),
    ("noMatchingRecipientInfo", 91),
    ("unsupportedKeyWrapAlgorithm", 92),
    ("badKeyTransRecipientInfo", 93),
    ("other", 127),
)


class ErrorCodeChoice(univ.Choice):
    pass


ErrorCodeChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType("enum", EnumeratedErrorCode()),
    namedtype.NamedType("oid", univ.ObjectIdentifier()),
)


class KeyPkgID(univ.OctetString):
    pass


class KeyPkgIdentifier(univ.Choice):
    pass


KeyPkgIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pkgID", KeyPkgID()),
    namedtype.NamedType("attribute", SingleAttribute()),
)


class KeyPkgVersion(univ.Integer):
    pass


KeyPkgVersion.namedValues = namedval.NamedValues(("v1", 1), ("v2", 2))

KeyPkgVersion.subtypeSpec = constraint.ValueRangeConstraint(1, 65535)


id_ct_KP_keyPackageError = univ.ObjectIdentifier("2.16.840.1.101.2.1.2.78.6")


class KeyPackageError(univ.Sequence):
    pass


KeyPackageError.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", KeyPkgVersion().subtype(value="v2")),
    namedtype.OptionalNamedType(
        "errorOf",
        KeyPkgIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType("errorBy", SIREntityName()),
    namedtype.NamedType("errorCode", ErrorCodeChoice()),
)


# Key Package Receipt CMS Content Type

id_ct_KP_keyPackageReceipt = univ.ObjectIdentifier("2.16.840.1.101.2.1.2.78.3")


class KeyPackageReceipt(univ.Sequence):
    pass


KeyPackageReceipt.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", KeyPkgVersion().subtype(value="v2")),
    namedtype.NamedType("receiptOf", KeyPkgIdentifier()),
    namedtype.NamedType("receivedBy", SIREntityName()),
)


# Key Package Receipt Request Attribute


class KeyPkgReceiptReq(univ.Sequence):
    pass


KeyPkgReceiptReq.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("encryptReceipt", univ.Boolean().subtype(value=0)),
    namedtype.OptionalNamedType(
        "receiptsFrom",
        SIREntityNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType("receiptsTo", SIREntityNames()),
)


id_aa_KP_keyPkgIdAndReceiptReq = univ.ObjectIdentifier("2.16.840.1.101.2.1.5.65")


class KeyPkgIdentifierAndReceiptReq(univ.Sequence):
    pass


KeyPkgIdentifierAndReceiptReq.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pkgID", KeyPkgID()),
    namedtype.OptionalNamedType("receiptReq", KeyPkgReceiptReq()),
)


# Map of Attribute Type OIDs to Attributes are added to
# the ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_KP_keyPkgIdAndReceiptReq: KeyPkgIdentifierAndReceiptReq(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)


# Map of CMC Content Type OIDs to CMC Content Types are added to
# the ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_KP_keyPackageError: KeyPackageError(),
    id_ct_KP_keyPackageReceipt: KeyPackageReceipt(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Additional Cryptographic Algorithms for Use with GOST 28147-89,
# GOST R 34.10-94, GOST R 34.10-2001, and GOST R 34.11-94 Algorithms
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4357.txt
# https://www.rfc-editor.org/errata/eid5927
# https://www.rfc-editor.org/errata/eid5928
#

from pyasn1.type import constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc5280

# Import from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Object Identifiers

id_CryptoPro = univ.ObjectIdentifier(
    (
        1,
        2,
        643,
        2,
        2,
    )
)


id_CryptoPro_modules = id_CryptoPro + (
    1,
    1,
)

id_CryptoPro_extensions = id_CryptoPro + (34,)

id_CryptoPro_policyIds = id_CryptoPro + (38,)

id_CryptoPro_policyQt = id_CryptoPro + (39,)


cryptographic_Gost_Useful_Definitions = id_CryptoPro_modules + (
    0,
    1,
)

gostR3411_94_DigestSyntax = id_CryptoPro_modules + (
    1,
    1,
)

gostR3410_94_PKISyntax = id_CryptoPro_modules + (
    2,
    1,
)

gostR3410_94_SignatureSyntax = id_CryptoPro_modules + (
    3,
    1,
)

gost28147_89_EncryptionSyntax = id_CryptoPro_modules + (
    4,
    1,
)

gostR3410_EncryptionSyntax = id_CryptoPro_modules + (
    5,
    2,
)

gost28147_89_ParamSetSyntax = id_CryptoPro_modules + (
    6,
    1,
)

gostR3411_94_ParamSetSyntax = id_CryptoPro_modules + (
    7,
    1,
)

gostR3410_94_ParamSetSyntax = id_CryptoPro_modules + (8, 1, 1)

gostR3410_2001_PKISyntax = id_CryptoPro_modules + (
    9,
    1,
)

gostR3410_2001_SignatureSyntax = id_CryptoPro_modules + (
    10,
    1,
)

gostR3410_2001_ParamSetSyntax = id_CryptoPro_modules + (
    12,
    1,
)

gost_CryptoPro_ExtendedKeyUsage = id_CryptoPro_modules + (
    13,
    1,
)

gost_CryptoPro_PrivateKey = id_CryptoPro_modules + (
    14,
    1,
)

gost_CryptoPro_PKIXCMP = id_CryptoPro_modules + (
    15,
    1,
)

gost_CryptoPro_TLS = id_CryptoPro_modules + (
    16,
    1,
)

gost_CryptoPro_Policy = id_CryptoPro_modules + (
    17,
    1,
)

gost_CryptoPro_Constants = id_CryptoPro_modules + (
    18,
    1,
)


id_CryptoPro_algorithms = id_CryptoPro

id_GostR3411_94_with_GostR3410_2001 = id_CryptoPro_algorithms + (3,)

id_GostR3411_94_with_GostR3410_94 = id_CryptoPro_algorithms + (4,)

id_GostR3411_94 = id_CryptoPro_algorithms + (9,)

id_Gost28147_89_None_KeyMeshing = id_CryptoPro_algorithms + (
    14,
    0,
)

id_Gost28147_89_CryptoPro_KeyMeshing = id_CryptoPro_algorithms + (
    14,
    1,
)

id_GostR3410_2001 = id_CryptoPro_algorithms + (19,)

id_GostR3410_94 = id_CryptoPro_algorithms + (20,)

id_Gost28147_89 = id_CryptoPro_algorithms + (21,)

id_Gost28147_89_MAC = id_CryptoPro_algorithms + (22,)

id_CryptoPro_hashes = id_CryptoPro_algorithms + (30,)

id_CryptoPro_encrypts = id_CryptoPro_algorithms + (31,)

id_CryptoPro_signs = id_CryptoPro_algorithms + (32,)

id_CryptoPro_exchanges = id_CryptoPro_algorithms + (33,)

id_CryptoPro_ecc_signs = id_CryptoPro_algorithms + (35,)

id_CryptoPro_ecc_exchanges = id_CryptoPro_algorithms + (36,)

id_CryptoPro_private_keys = id_CryptoPro_algorithms + (37,)

id_CryptoPro_pkixcmp_infos = id_CryptoPro_algorithms + (41,)

id_CryptoPro_audit_service_types = id_CryptoPro_algorithms + (42,)

id_CryptoPro_audit_record_types = id_CryptoPro_algorithms + (43,)

id_CryptoPro_attributes = id_CryptoPro_algorithms + (44,)

id_CryptoPro_name_service_types = id_CryptoPro_algorithms + (45,)

id_GostR3410_2001DH = id_CryptoPro_algorithms + (98,)

id_GostR3410_94DH = id_CryptoPro_algorithms + (99,)


id_Gost28147_89_TestParamSet = id_CryptoPro_encrypts + (0,)

id_Gost28147_89_CryptoPro_A_ParamSet = id_CryptoPro_encrypts + (1,)

id_Gost28147_89_CryptoPro_B_ParamSet = id_CryptoPro_encrypts + (2,)

id_Gost28147_89_CryptoPro_C_ParamSet = id_CryptoPro_encrypts + (3,)

id_Gost28147_89_CryptoPro_D_ParamSet = id_CryptoPro_encrypts + (4,)

id_Gost28147_89_CryptoPro_Oscar_1_1_ParamSet = id_CryptoPro_encrypts + (5,)

id_Gost28147_89_CryptoPro_Oscar_1_0_ParamSet = id_CryptoPro_encrypts + (6,)

id_Gost28147_89_CryptoPro_RIC_1_ParamSet = id_CryptoPro_encrypts + (7,)


id_GostR3410_2001_TestParamSet = id_CryptoPro_ecc_signs + (0,)

id_GostR3410_2001_CryptoPro_A_ParamSet = id_CryptoPro_ecc_signs + (1,)

id_GostR3410_2001_CryptoPro_B_ParamSet = id_CryptoPro_ecc_signs + (2,)

id_GostR3410_2001_CryptoPro_C_ParamSet = id_CryptoPro_ecc_signs + (3,)


id_GostR3410_2001_CryptoPro_XchA_ParamSet = id_CryptoPro_ecc_exchanges + (0,)

id_GostR3410_2001_CryptoPro_XchB_ParamSet = id_CryptoPro_ecc_exchanges + (1,)


id_GostR3410_94_TestParamSet = id_CryptoPro_signs + (0,)

id_GostR3410_94_CryptoPro_A_ParamSet = id_CryptoPro_signs + (2,)

id_GostR3410_94_CryptoPro_B_ParamSet = id_CryptoPro_signs + (3,)

id_GostR3410_94_CryptoPro_C_ParamSet = id_CryptoPro_signs + (4,)

id_GostR3410_94_CryptoPro_D_ParamSet = id_CryptoPro_signs + (5,)


id_GostR3410_94_CryptoPro_XchA_ParamSet = id_CryptoPro_exchanges + (1,)

id_GostR3410_94_CryptoPro_XchB_ParamSet = id_CryptoPro_exchanges + (2,)

id_GostR3410_94_CryptoPro_XchC_ParamSet = id_CryptoPro_exchanges + (3,)


id_GostR3410_94_a = id_GostR3410_94 + (1,)

id_GostR3410_94_aBis = id_GostR3410_94 + (2,)

id_GostR3410_94_b = id_GostR3410_94 + (3,)

id_GostR3410_94_bBis = id_GostR3410_94 + (4,)


id_GostR3411_94_TestParamSet = id_CryptoPro_hashes + (0,)

id_GostR3411_94_CryptoProParamSet = id_CryptoPro_hashes + (1,)


class Gost28147_89_ParamSet(univ.ObjectIdentifier):
    pass


Gost28147_89_ParamSet.subtypeSpec = constraint.SingleValueConstraint(
    id_Gost28147_89_TestParamSet,
    id_Gost28147_89_CryptoPro_A_ParamSet,
    id_Gost28147_89_CryptoPro_B_ParamSet,
    id_Gost28147_89_CryptoPro_C_ParamSet,
    id_Gost28147_89_CryptoPro_D_ParamSet,
    id_Gost28147_89_CryptoPro_Oscar_1_1_ParamSet,
    id_Gost28147_89_CryptoPro_Oscar_1_0_ParamSet,
    id_Gost28147_89_CryptoPro_RIC_1_ParamSet,
)


class Gost28147_89_BlobParameters(univ.Sequence):
    pass


Gost28147_89_BlobParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("encryptionParamSet", Gost28147_89_ParamSet())
)


class Gost28147_89_MAC(univ.OctetString):
    pass


Gost28147_89_MAC.subtypeSpec = constraint.ValueSizeConstraint(1, 4)


class Gost28147_89_Key(univ.OctetString):
    pass


Gost28147_89_Key.subtypeSpec = constraint.ValueSizeConstraint(32, 32)


class Gost28147_89_EncryptedKey(univ.Sequence):
    pass


Gost28147_89_EncryptedKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("encryptedKey", Gost28147_89_Key()),
    namedtype.OptionalNamedType(
        "maskKey",
        Gost28147_89_Key().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType("macKey", Gost28147_89_MAC()),
)


class Gost28147_89_IV(univ.OctetString):
    pass


Gost28147_89_IV.subtypeSpec = constraint.ValueSizeConstraint(8, 8)


class Gost28147_89_UZ(univ.OctetString):
    pass


Gost28147_89_UZ.subtypeSpec = constraint.ValueSizeConstraint(64, 64)


class Gost28147_89_ParamSetParameters(univ.Sequence):
    pass


Gost28147_89_ParamSetParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("eUZ", Gost28147_89_UZ()),
    namedtype.NamedType(
        "mode",
        univ.Integer(
            namedValues=namedval.NamedValues(
                ("gost28147-89-CNT", 0), ("gost28147-89-CFB", 1), ("cryptoPro-CBC", 2)
            )
        ),
    ),
    namedtype.NamedType(
        "shiftBits",
        univ.Integer(namedValues=namedval.NamedValues(("gost28147-89-block", 64))),
    ),
    namedtype.NamedType("keyMeshing", AlgorithmIdentifier()),
)


class Gost28147_89_Parameters(univ.Sequence):
    pass


Gost28147_89_Parameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("iv", Gost28147_89_IV()),
    namedtype.NamedType("encryptionParamSet", Gost28147_89_ParamSet()),
)


class GostR3410_2001_CertificateSignature(univ.BitString):
    pass


GostR3410_2001_CertificateSignature.subtypeSpec = constraint.ValueSizeConstraint(
    256, 512
)


class GostR3410_2001_ParamSetParameters(univ.Sequence):
    pass


GostR3410_2001_ParamSetParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("a", univ.Integer()),
    namedtype.NamedType("b", univ.Integer()),
    namedtype.NamedType("p", univ.Integer()),
    namedtype.NamedType("q", univ.Integer()),
    namedtype.NamedType("x", univ.Integer()),
    namedtype.NamedType("y", univ.Integer()),
)


class GostR3410_2001_PublicKey(univ.OctetString):
    pass


GostR3410_2001_PublicKey.subtypeSpec = constraint.ValueSizeConstraint(64, 64)


class GostR3410_2001_PublicKeyParameters(univ.Sequence):
    pass


GostR3410_2001_PublicKeyParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "publicKeyParamSet",
        univ.ObjectIdentifier().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                id_GostR3410_2001_TestParamSet,
                id_GostR3410_2001_CryptoPro_A_ParamSet,
                id_GostR3410_2001_CryptoPro_B_ParamSet,
                id_GostR3410_2001_CryptoPro_C_ParamSet,
                id_GostR3410_2001_CryptoPro_XchA_ParamSet,
                id_GostR3410_2001_CryptoPro_XchB_ParamSet,
            )
        ),
    ),
    namedtype.NamedType(
        "digestParamSet",
        univ.ObjectIdentifier().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                id_GostR3411_94_TestParamSet, id_GostR3411_94_CryptoProParamSet
            )
        ),
    ),
    namedtype.DefaultedNamedType(
        "encryptionParamSet",
        Gost28147_89_ParamSet().subtype(value=id_Gost28147_89_CryptoPro_A_ParamSet),
    ),
)


class GostR3410_94_CertificateSignature(univ.BitString):
    pass


GostR3410_94_CertificateSignature.subtypeSpec = constraint.ValueSizeConstraint(256, 512)


class GostR3410_94_ParamSetParameters_t(univ.Integer):
    pass


GostR3410_94_ParamSetParameters_t.subtypeSpec = constraint.SingleValueConstraint(
    512, 1024
)


class GostR3410_94_ParamSetParameters(univ.Sequence):
    pass


GostR3410_94_ParamSetParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("t", GostR3410_94_ParamSetParameters_t()),
    namedtype.NamedType("p", univ.Integer()),
    namedtype.NamedType("q", univ.Integer()),
    namedtype.NamedType("a", univ.Integer()),
    namedtype.OptionalNamedType("validationAlgorithm", AlgorithmIdentifier()),
)


class GostR3410_94_PublicKey(univ.OctetString):
    pass


GostR3410_94_PublicKey.subtypeSpec = constraint.ConstraintsUnion(
    constraint.ValueSizeConstraint(64, 64), constraint.ValueSizeConstraint(128, 128)
)


class GostR3410_94_PublicKeyParameters(univ.Sequence):
    pass


GostR3410_94_PublicKeyParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "publicKeyParamSet",
        univ.ObjectIdentifier().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                id_GostR3410_94_TestParamSet,
                id_GostR3410_94_CryptoPro_A_ParamSet,
                id_GostR3410_94_CryptoPro_B_ParamSet,
                id_GostR3410_94_CryptoPro_C_ParamSet,
                id_GostR3410_94_CryptoPro_D_ParamSet,
                id_GostR3410_94_CryptoPro_XchA_ParamSet,
                id_GostR3410_94_CryptoPro_XchB_ParamSet,
                id_GostR3410_94_CryptoPro_XchC_ParamSet,
            )
        ),
    ),
    namedtype.NamedType(
        "digestParamSet",
        univ.ObjectIdentifier().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                id_GostR3411_94_TestParamSet, id_GostR3411_94_CryptoProParamSet
            )
        ),
    ),
    namedtype.DefaultedNamedType(
        "encryptionParamSet",
        Gost28147_89_ParamSet().subtype(value=id_Gost28147_89_CryptoPro_A_ParamSet),
    ),
)


class GostR3410_94_ValidationBisParameters_c(univ.Integer):
    pass


GostR3410_94_ValidationBisParameters_c.subtypeSpec = constraint.ValueRangeConstraint(
    0, 4294967295
)


class GostR3410_94_ValidationBisParameters(univ.Sequence):
    pass


GostR3410_94_ValidationBisParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("x0", GostR3410_94_ValidationBisParameters_c()),
    namedtype.NamedType("c", GostR3410_94_ValidationBisParameters_c()),
    namedtype.OptionalNamedType("d", univ.Integer()),
)


class GostR3410_94_ValidationParameters_c(univ.Integer):
    pass


GostR3410_94_ValidationParameters_c.subtypeSpec = constraint.ValueRangeConstraint(
    0, 65535
)


class GostR3410_94_ValidationParameters(univ.Sequence):
    pass


GostR3410_94_ValidationParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("x0", GostR3410_94_ValidationParameters_c()),
    namedtype.NamedType("c", GostR3410_94_ValidationParameters_c()),
    namedtype.OptionalNamedType("d", univ.Integer()),
)


class GostR3411_94_Digest(univ.OctetString):
    pass


GostR3411_94_Digest.subtypeSpec = constraint.ValueSizeConstraint(32, 32)


class GostR3411_94_DigestParameters(univ.ObjectIdentifier):
    pass


GostR3411_94_DigestParameters.subtypeSpec = constraint.ConstraintsUnion(
    constraint.SingleValueConstraint(id_GostR3411_94_TestParamSet),
    constraint.SingleValueConstraint(id_GostR3411_94_CryptoProParamSet),
)


class GostR3411_94_ParamSetParameters(univ.Sequence):
    pass


GostR3411_94_ParamSetParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hUZ", Gost28147_89_UZ()),
    namedtype.NamedType("h0", GostR3411_94_Digest()),
)


# Update the Algorithm Identifier map in rfc5280.py

_algorithmIdentifierMapUpdate = {
    id_Gost28147_89: Gost28147_89_Parameters(),
    id_Gost28147_89_TestParamSet: Gost28147_89_ParamSetParameters(),
    id_Gost28147_89_CryptoPro_A_ParamSet: Gost28147_89_ParamSetParameters(),
    id_Gost28147_89_CryptoPro_B_ParamSet: Gost28147_89_ParamSetParameters(),
    id_Gost28147_89_CryptoPro_C_ParamSet: Gost28147_89_ParamSetParameters(),
    id_Gost28147_89_CryptoPro_D_ParamSet: Gost28147_89_ParamSetParameters(),
    id_Gost28147_89_CryptoPro_KeyMeshing: univ.Null(""),
    id_Gost28147_89_None_KeyMeshing: univ.Null(""),
    id_GostR3410_94: GostR3410_94_PublicKeyParameters(),
    id_GostR3410_94_TestParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_A_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_B_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_C_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_D_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_XchA_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_XchB_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_CryptoPro_XchC_ParamSet: GostR3410_94_ParamSetParameters(),
    id_GostR3410_94_a: GostR3410_94_ValidationParameters(),
    id_GostR3410_94_aBis: GostR3410_94_ValidationBisParameters(),
    id_GostR3410_94_b: GostR3410_94_ValidationParameters(),
    id_GostR3410_94_bBis: GostR3410_94_ValidationBisParameters(),
    id_GostR3410_2001: univ.Null(""),
    id_GostR3411_94: univ.Null(""),
    id_GostR3411_94_TestParamSet: GostR3411_94_ParamSetParameters(),
    id_GostR3411_94_CryptoProParamSet: GostR3411_94_ParamSetParameters(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Elliptic Curve Private Key
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5915.txt

from pyasn1.type import namedtype, namedval, tag, univ

from pyasn1_modules import rfc5480


class ECPrivateKey(univ.Sequence):
    pass


ECPrivateKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "version", univ.Integer(namedValues=namedval.NamedValues(("ecPrivkeyVer1", 1)))
    ),
    namedtype.NamedType("privateKey", univ.OctetString()),
    namedtype.OptionalNamedType(
        "parameters",
        rfc5480.ECParameters().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "publicKey",
        univ.BitString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv2c PDU syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc1905.txt
#
from pyasn1.type import constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc1902

max_bindings = rfc1902.Integer(2147483647)


class _BindValue(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("value", rfc1902.ObjectSyntax()),
        namedtype.NamedType("unSpecified", univ.Null()),
        namedtype.NamedType(
            "noSuchObject",
            univ.Null().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "noSuchInstance",
            univ.Null().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "endOfMibView",
            univ.Null().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class VarBind(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("name", rfc1902.ObjectName()),
        namedtype.NamedType("", _BindValue()),
    )


class VarBindList(univ.SequenceOf):
    componentType = VarBind()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(
        0, max_bindings
    )


class PDU(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("request-id", rfc1902.Integer32()),
        namedtype.NamedType(
            "error-status",
            univ.Integer(
                namedValues=namedval.NamedValues(
                    ("noError", 0),
                    ("tooBig", 1),
                    ("noSuchName", 2),
                    ("badValue", 3),
                    ("readOnly", 4),
                    ("genErr", 5),
                    ("noAccess", 6),
                    ("wrongType", 7),
                    ("wrongLength", 8),
                    ("wrongEncoding", 9),
                    ("wrongValue", 10),
                    ("noCreation", 11),
                    ("inconsistentValue", 12),
                    ("resourceUnavailable", 13),
                    ("commitFailed", 14),
                    ("undoFailed", 15),
                    ("authorizationError", 16),
                    ("notWritable", 17),
                    ("inconsistentName", 18),
                )
            ),
        ),
        namedtype.NamedType(
            "error-index",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, max_bindings)
            ),
        ),
        namedtype.NamedType("variable-bindings", VarBindList()),
    )


class BulkPDU(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("request-id", rfc1902.Integer32()),
        namedtype.NamedType(
            "non-repeaters",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, max_bindings)
            ),
        ),
        namedtype.NamedType(
            "max-repetitions",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, max_bindings)
            ),
        ),
        namedtype.NamedType("variable-bindings", VarBindList()),
    )


class GetRequestPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
    )


class GetNextRequestPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
    )


class ResponsePDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
    )


class SetRequestPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
    )


class GetBulkRequestPDU(BulkPDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
    )


class InformRequestPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
    )


class SNMPv2TrapPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 7)
    )


class ReportPDU(PDU):
    tagSet = PDU.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 8)
    )


class PDUs(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("get-request", GetRequestPDU()),
        namedtype.NamedType("get-next-request", GetNextRequestPDU()),
        namedtype.NamedType("get-bulk-request", GetBulkRequestPDU()),
        namedtype.NamedType("response", ResponsePDU()),
        namedtype.NamedType("set-request", SetRequestPDU()),
        namedtype.NamedType("inform-request", InformRequestPDU()),
        namedtype.NamedType("snmpV2-trap", SNMPv2TrapPDU()),
        namedtype.NamedType("report", ReportPDU()),
    )



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Elliptic Curve Cryptography (ECC) Algorithms in the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5753.txt
#

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc5280, rfc5480, rfc5652, rfc5751, rfc8018

# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Imports from RFC 5652

OriginatorPublicKey = rfc5652.OriginatorPublicKey

UserKeyingMaterial = rfc5652.UserKeyingMaterial


# Imports from RFC 5480

ECDSA_Sig_Value = rfc5480.ECDSA_Sig_Value

ECParameters = rfc5480.ECParameters

ECPoint = rfc5480.ECPoint

id_ecPublicKey = rfc5480.id_ecPublicKey


# Imports from RFC 8018

id_hmacWithSHA224 = rfc8018.id_hmacWithSHA224

id_hmacWithSHA256 = rfc8018.id_hmacWithSHA256

id_hmacWithSHA384 = rfc8018.id_hmacWithSHA384

id_hmacWithSHA512 = rfc8018.id_hmacWithSHA512


# Object Identifier arcs

x9_63_scheme = univ.ObjectIdentifier("1.3.133.16.840.63.0")

secg_scheme = univ.ObjectIdentifier("1.3.132.1")


# Object Identifiers for the algorithms

dhSinglePass_cofactorDH_sha1kdf_scheme = x9_63_scheme + (3,)

dhSinglePass_cofactorDH_sha224kdf_scheme = secg_scheme + (
    14,
    0,
)

dhSinglePass_cofactorDH_sha256kdf_scheme = secg_scheme + (
    14,
    1,
)

dhSinglePass_cofactorDH_sha384kdf_scheme = secg_scheme + (
    14,
    2,
)

dhSinglePass_cofactorDH_sha512kdf_scheme = secg_scheme + (
    14,
    3,
)

dhSinglePass_stdDH_sha1kdf_scheme = x9_63_scheme + (2,)

dhSinglePass_stdDH_sha224kdf_scheme = secg_scheme + (
    11,
    0,
)

dhSinglePass_stdDH_sha256kdf_scheme = secg_scheme + (
    11,
    1,
)

dhSinglePass_stdDH_sha384kdf_scheme = secg_scheme + (
    11,
    2,
)

dhSinglePass_stdDH_sha512kdf_scheme = secg_scheme + (
    11,
    3,
)

mqvSinglePass_sha1kdf_scheme = x9_63_scheme + (16,)

mqvSinglePass_sha224kdf_scheme = secg_scheme + (
    15,
    0,
)

mqvSinglePass_sha256kdf_scheme = secg_scheme + (
    15,
    1,
)

mqvSinglePass_sha384kdf_scheme = secg_scheme + (
    15,
    2,
)

mqvSinglePass_sha512kdf_scheme = secg_scheme + (
    15,
    3,
)


# Structures for parameters and key derivation


class IV(univ.OctetString):
    # Exactly 8 octets
    pass


class CBCParameter(IV):
    pass


class KeyWrapAlgorithm(AlgorithmIdentifier):
    pass


class ECC_CMS_SharedInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("keyInfo", KeyWrapAlgorithm()),
        namedtype.OptionalNamedType(
            "entityUInfo",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "suppPubInfo",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class MQVuserKeyingMaterial(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("ephemeralPublicKey", OriginatorPublicKey()),
        namedtype.OptionalNamedType(
            "addedukm",
            UserKeyingMaterial().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


# Update the Algorithm Identifier map in rfc5280.py and
# Update the SMIMECapabilities Attribute Map in rfc5751.py

_algorithmIdentifierMapUpdate = {
    dhSinglePass_stdDH_sha1kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_stdDH_sha224kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_stdDH_sha256kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_stdDH_sha384kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_stdDH_sha512kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_cofactorDH_sha1kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_cofactorDH_sha224kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_cofactorDH_sha256kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_cofactorDH_sha384kdf_scheme: KeyWrapAlgorithm(),
    dhSinglePass_cofactorDH_sha512kdf_scheme: KeyWrapAlgorithm(),
    mqvSinglePass_sha1kdf_scheme: KeyWrapAlgorithm(),
    mqvSinglePass_sha224kdf_scheme: KeyWrapAlgorithm(),
    mqvSinglePass_sha256kdf_scheme: KeyWrapAlgorithm(),
    mqvSinglePass_sha384kdf_scheme: KeyWrapAlgorithm(),
    mqvSinglePass_sha512kdf_scheme: KeyWrapAlgorithm(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)

rfc5751.smimeCapabilityMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc5751, rfc8692


class AlgorithmIdentifierTestCase(unittest.TestCase):
    pem_text = """\
MEowCwYJYIZIAWUDBAILMAsGCWCGSAFlAwQCDDAKBggrBgEFBQcGHjAKBggrBgEF
BQcGHzAKBggrBgEFBQcGIDAKBggrBgEFBQcGIQ==
"""

    def setUp(self):
        self.asn1Spec = rfc5751.SMIMECapabilities()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        oid_list = (
            rfc8692.id_shake128,
            rfc8692.id_shake256,
            rfc8692.id_RSASSA_PSS_SHAKE128,
            rfc8692.id_RSASSA_PSS_SHAKE256,
            rfc8692.id_ecdsa_with_shake128,
            rfc8692.id_ecdsa_with_shake256,
        )

        count = 0
        for algid in asn1Object:
            self.assertTrue(algid["capabilityID"] in oid_list)
            count += 1

        self.assertTrue(len(oid_list), count)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# The scrypt Password-Based Key Derivation Function
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8520.txt
# https://www.rfc-editor.org/errata/eid5871
#

from pyasn1.type import constraint, namedtype, univ

from pyasn1_modules import rfc5280

MAX = float("inf")


id_scrypt = univ.ObjectIdentifier("1.3.6.1.4.1.11591.4.11")


class Scrypt_params(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("salt", univ.OctetString()),
        namedtype.NamedType(
            "costParameter",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
        namedtype.NamedType(
            "blockSize",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
        namedtype.NamedType(
            "parallelizationParameter",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
        namedtype.OptionalNamedType(
            "keyLength",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
    )


# Update the Algorithm Identifier map in rfc5280.py

_algorithmIdentifierMapUpdate = {
    id_scrypt: Scrypt_params(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Stanisław Pitucha with asn1ate tool.
# Modified by Russ Housley to add support for opentypes.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc5652.txt
#
from pyasn1.type import constraint, namedtype, namedval, opentype, tag, univ, useful

from pyasn1_modules import rfc3281, rfc5280

MAX = float("inf")


def _buildOid(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


cmsContentTypesMap = {}

cmsAttributesMap = {}

otherKeyAttributesMap = {}

otherCertFormatMap = {}

otherRevInfoFormatMap = {}

otherRecipientInfoMap = {}


class AttCertVersionV1(univ.Integer):
    pass


AttCertVersionV1.namedValues = namedval.NamedValues(("v1", 0))


class AttributeCertificateInfoV1(univ.Sequence):
    pass


AttributeCertificateInfoV1.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", AttCertVersionV1().subtype(value="v1")),
    namedtype.NamedType(
        "subject",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "baseCertificateID",
                    rfc3281.IssuerSerial().subtype(
                        explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
                    ),
                ),
                namedtype.NamedType(
                    "subjectName",
                    rfc5280.GeneralNames().subtype(
                        explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
                    ),
                ),
            )
        ),
    ),
    namedtype.NamedType("issuer", rfc5280.GeneralNames()),
    namedtype.NamedType("signature", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("serialNumber", rfc5280.CertificateSerialNumber()),
    namedtype.NamedType("attCertValidityPeriod", rfc3281.AttCertValidityPeriod()),
    namedtype.NamedType(
        "attributes", univ.SequenceOf(componentType=rfc5280.Attribute())
    ),
    namedtype.OptionalNamedType("issuerUniqueID", rfc5280.UniqueIdentifier()),
    namedtype.OptionalNamedType("extensions", rfc5280.Extensions()),
)


class AttributeCertificateV1(univ.Sequence):
    pass


AttributeCertificateV1.componentType = namedtype.NamedTypes(
    namedtype.NamedType("acInfo", AttributeCertificateInfoV1()),
    namedtype.NamedType("signatureAlgorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("signature", univ.BitString()),
)


class AttributeValue(univ.Any):
    pass


class Attribute(univ.Sequence):
    pass


Attribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "attrValues",
        univ.SetOf(componentType=AttributeValue()),
        openType=opentype.OpenType("attrType", cmsAttributesMap),
    ),
)


class SignedAttributes(univ.SetOf):
    pass


SignedAttributes.componentType = Attribute()
SignedAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class AttributeCertificateV2(rfc3281.AttributeCertificate):
    pass


class OtherKeyAttribute(univ.Sequence):
    pass


OtherKeyAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyAttrId", univ.ObjectIdentifier()),
    namedtype.OptionalNamedType(
        "keyAttr",
        univ.Any(),
        openType=opentype.OpenType("keyAttrId", otherKeyAttributesMap),
    ),
)


class UnauthAttributes(univ.SetOf):
    pass


UnauthAttributes.componentType = Attribute()
UnauthAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_encryptedData = _buildOid(1, 2, 840, 113549, 1, 7, 6)


class SignatureValue(univ.OctetString):
    pass


class IssuerAndSerialNumber(univ.Sequence):
    pass


IssuerAndSerialNumber.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuer", rfc5280.Name()),
    namedtype.NamedType("serialNumber", rfc5280.CertificateSerialNumber()),
)


class SubjectKeyIdentifier(univ.OctetString):
    pass


class RecipientKeyIdentifier(univ.Sequence):
    pass


RecipientKeyIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("subjectKeyIdentifier", SubjectKeyIdentifier()),
    namedtype.OptionalNamedType("date", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("other", OtherKeyAttribute()),
)


class KeyAgreeRecipientIdentifier(univ.Choice):
    pass


KeyAgreeRecipientIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
    namedtype.NamedType(
        "rKeyId",
        RecipientKeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
)


class EncryptedKey(univ.OctetString):
    pass


class RecipientEncryptedKey(univ.Sequence):
    pass


RecipientEncryptedKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("rid", KeyAgreeRecipientIdentifier()),
    namedtype.NamedType("encryptedKey", EncryptedKey()),
)


class RecipientEncryptedKeys(univ.SequenceOf):
    pass


RecipientEncryptedKeys.componentType = RecipientEncryptedKey()


class MessageAuthenticationCode(univ.OctetString):
    pass


class CMSVersion(univ.Integer):
    pass


CMSVersion.namedValues = namedval.NamedValues(
    ("v0", 0), ("v1", 1), ("v2", 2), ("v3", 3), ("v4", 4), ("v5", 5)
)


class OtherCertificateFormat(univ.Sequence):
    pass


OtherCertificateFormat.componentType = namedtype.NamedTypes(
    namedtype.NamedType("otherCertFormat", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "otherCert",
        univ.Any(),
        openType=opentype.OpenType("otherCertFormat", otherCertFormatMap),
    ),
)


class ExtendedCertificateInfo(univ.Sequence):
    pass


ExtendedCertificateInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("certificate", rfc5280.Certificate()),
    namedtype.NamedType("attributes", UnauthAttributes()),
)


class Signature(univ.BitString):
    pass


class SignatureAlgorithmIdentifier(rfc5280.AlgorithmIdentifier):
    pass


class ExtendedCertificate(univ.Sequence):
    pass


ExtendedCertificate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("extendedCertificateInfo", ExtendedCertificateInfo()),
    namedtype.NamedType("signatureAlgorithm", SignatureAlgorithmIdentifier()),
    namedtype.NamedType("signature", Signature()),
)


class CertificateChoices(univ.Choice):
    pass


CertificateChoices.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certificate", rfc5280.Certificate()),
    namedtype.NamedType(
        "extendedCertificate",
        ExtendedCertificate().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "v1AttrCert",
        AttributeCertificateV1().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "v2AttrCert",
        AttributeCertificateV2().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "other",
        OtherCertificateFormat().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
)


class CertificateSet(univ.SetOf):
    pass


CertificateSet.componentType = CertificateChoices()


class OtherRevocationInfoFormat(univ.Sequence):
    pass


OtherRevocationInfoFormat.componentType = namedtype.NamedTypes(
    namedtype.NamedType("otherRevInfoFormat", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "otherRevInfo",
        univ.Any(),
        openType=opentype.OpenType("otherRevInfoFormat", otherRevInfoFormatMap),
    ),
)


class RevocationInfoChoice(univ.Choice):
    pass


RevocationInfoChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType("crl", rfc5280.CertificateList()),
    namedtype.NamedType(
        "other",
        OtherRevocationInfoFormat().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class RevocationInfoChoices(univ.SetOf):
    pass


RevocationInfoChoices.componentType = RevocationInfoChoice()


class OriginatorInfo(univ.Sequence):
    pass


OriginatorInfo.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "certs",
        CertificateSet().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "crls",
        RevocationInfoChoices().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class ContentType(univ.ObjectIdentifier):
    pass


class EncryptedContent(univ.OctetString):
    pass


class ContentEncryptionAlgorithmIdentifier(rfc5280.AlgorithmIdentifier):
    pass


class EncryptedContentInfo(univ.Sequence):
    pass


EncryptedContentInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("contentType", ContentType()),
    namedtype.NamedType(
        "contentEncryptionAlgorithm", ContentEncryptionAlgorithmIdentifier()
    ),
    namedtype.OptionalNamedType(
        "encryptedContent",
        EncryptedContent().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class UnprotectedAttributes(univ.SetOf):
    pass


UnprotectedAttributes.componentType = Attribute()
UnprotectedAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class KeyEncryptionAlgorithmIdentifier(rfc5280.AlgorithmIdentifier):
    pass


class KEKIdentifier(univ.Sequence):
    pass


KEKIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyIdentifier", univ.OctetString()),
    namedtype.OptionalNamedType("date", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("other", OtherKeyAttribute()),
)


class KEKRecipientInfo(univ.Sequence):
    pass


KEKRecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("kekid", KEKIdentifier()),
    namedtype.NamedType("keyEncryptionAlgorithm", KeyEncryptionAlgorithmIdentifier()),
    namedtype.NamedType("encryptedKey", EncryptedKey()),
)


class KeyDerivationAlgorithmIdentifier(rfc5280.AlgorithmIdentifier):
    pass


class PasswordRecipientInfo(univ.Sequence):
    pass


PasswordRecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.OptionalNamedType(
        "keyDerivationAlgorithm",
        KeyDerivationAlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType("keyEncryptionAlgorithm", KeyEncryptionAlgorithmIdentifier()),
    namedtype.NamedType("encryptedKey", EncryptedKey()),
)


class RecipientIdentifier(univ.Choice):
    pass


RecipientIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
    namedtype.NamedType(
        "subjectKeyIdentifier",
        SubjectKeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class KeyTransRecipientInfo(univ.Sequence):
    pass


KeyTransRecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("rid", RecipientIdentifier()),
    namedtype.NamedType("keyEncryptionAlgorithm", KeyEncryptionAlgorithmIdentifier()),
    namedtype.NamedType("encryptedKey", EncryptedKey()),
)


class UserKeyingMaterial(univ.OctetString):
    pass


class OriginatorPublicKey(univ.Sequence):
    pass


OriginatorPublicKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("algorithm", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("publicKey", univ.BitString()),
)


class OriginatorIdentifierOrKey(univ.Choice):
    pass


OriginatorIdentifierOrKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
    namedtype.NamedType(
        "subjectKeyIdentifier",
        SubjectKeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "originatorKey",
        OriginatorPublicKey().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class KeyAgreeRecipientInfo(univ.Sequence):
    pass


KeyAgreeRecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType(
        "originator",
        OriginatorIdentifierOrKey().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "ukm",
        UserKeyingMaterial().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType("keyEncryptionAlgorithm", KeyEncryptionAlgorithmIdentifier()),
    namedtype.NamedType("recipientEncryptedKeys", RecipientEncryptedKeys()),
)


class OtherRecipientInfo(univ.Sequence):
    pass


OtherRecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("oriType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "oriValue",
        univ.Any(),
        openType=opentype.OpenType("oriType", otherRecipientInfoMap),
    ),
)


class RecipientInfo(univ.Choice):
    pass


RecipientInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("ktri", KeyTransRecipientInfo()),
    namedtype.NamedType(
        "kari",
        KeyAgreeRecipientInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
    namedtype.NamedType(
        "kekri",
        KEKRecipientInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
    namedtype.NamedType(
        "pwri",
        PasswordRecipientInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
    namedtype.NamedType(
        "ori",
        OtherRecipientInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
        ),
    ),
)


class RecipientInfos(univ.SetOf):
    pass


RecipientInfos.componentType = RecipientInfo()
RecipientInfos.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class EnvelopedData(univ.Sequence):
    pass


EnvelopedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.OptionalNamedType(
        "originatorInfo",
        OriginatorInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType("recipientInfos", RecipientInfos()),
    namedtype.NamedType("encryptedContentInfo", EncryptedContentInfo()),
    namedtype.OptionalNamedType(
        "unprotectedAttrs",
        UnprotectedAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class DigestAlgorithmIdentifier(rfc5280.AlgorithmIdentifier):
    pass


id_ct_contentInfo = _buildOid(1, 2, 840, 113549, 1, 9, 16, 1, 6)

id_digestedData = _buildOid(1, 2, 840, 113549, 1, 7, 5)


class EncryptedData(univ.Sequence):
    pass


EncryptedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("encryptedContentInfo", EncryptedContentInfo()),
    namedtype.OptionalNamedType(
        "unprotectedAttrs",
        UnprotectedAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)

id_messageDigest = _buildOid(1, 2, 840, 113549, 1, 9, 4)

id_signedData = _buildOid(1, 2, 840, 113549, 1, 7, 2)


class MessageAuthenticationCodeAlgorithm(rfc5280.AlgorithmIdentifier):
    pass


class UnsignedAttributes(univ.SetOf):
    pass


UnsignedAttributes.componentType = Attribute()
UnsignedAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class SignerIdentifier(univ.Choice):
    pass


SignerIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
    namedtype.NamedType(
        "subjectKeyIdentifier",
        SubjectKeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class SignerInfo(univ.Sequence):
    pass


SignerInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("sid", SignerIdentifier()),
    namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
    namedtype.OptionalNamedType(
        "signedAttrs",
        SignedAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType("signatureAlgorithm", SignatureAlgorithmIdentifier()),
    namedtype.NamedType("signature", SignatureValue()),
    namedtype.OptionalNamedType(
        "unsignedAttrs",
        UnsignedAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class SignerInfos(univ.SetOf):
    pass


SignerInfos.componentType = SignerInfo()


class Countersignature(SignerInfo):
    pass


class ContentInfo(univ.Sequence):
    pass


ContentInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("contentType", ContentType()),
    namedtype.NamedType(
        "content",
        univ.Any().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
        openType=opentype.OpenType("contentType", cmsContentTypesMap),
    ),
)


class EncapsulatedContentInfo(univ.Sequence):
    pass


EncapsulatedContentInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("eContentType", ContentType()),
    namedtype.OptionalNamedType(
        "eContent",
        univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)

id_countersignature = _buildOid(1, 2, 840, 113549, 1, 9, 6)

id_data = _buildOid(1, 2, 840, 113549, 1, 7, 1)


class MessageDigest(univ.OctetString):
    pass


class AuthAttributes(univ.SetOf):
    pass


AuthAttributes.componentType = Attribute()
AuthAttributes.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class Time(univ.Choice):
    pass


Time.componentType = namedtype.NamedTypes(
    namedtype.NamedType("utcTime", useful.UTCTime()),
    namedtype.NamedType("generalTime", useful.GeneralizedTime()),
)


class AuthenticatedData(univ.Sequence):
    pass


AuthenticatedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.OptionalNamedType(
        "originatorInfo",
        OriginatorInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType("recipientInfos", RecipientInfos()),
    namedtype.NamedType("macAlgorithm", MessageAuthenticationCodeAlgorithm()),
    namedtype.OptionalNamedType(
        "digestAlgorithm",
        DigestAlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType("encapContentInfo", EncapsulatedContentInfo()),
    namedtype.OptionalNamedType(
        "authAttrs",
        AuthAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType("mac", MessageAuthenticationCode()),
    namedtype.OptionalNamedType(
        "unauthAttrs",
        UnauthAttributes().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
)

id_contentType = _buildOid(1, 2, 840, 113549, 1, 9, 3)


class ExtendedCertificateOrCertificate(univ.Choice):
    pass


ExtendedCertificateOrCertificate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certificate", rfc5280.Certificate()),
    namedtype.NamedType(
        "extendedCertificate",
        ExtendedCertificate().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
)


class Digest(univ.OctetString):
    pass


class DigestedData(univ.Sequence):
    pass


DigestedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
    namedtype.NamedType("encapContentInfo", EncapsulatedContentInfo()),
    namedtype.NamedType("digest", Digest()),
)

id_envelopedData = _buildOid(1, 2, 840, 113549, 1, 7, 3)


class DigestAlgorithmIdentifiers(univ.SetOf):
    pass


DigestAlgorithmIdentifiers.componentType = DigestAlgorithmIdentifier()


class SignedData(univ.Sequence):
    pass


SignedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", CMSVersion()),
    namedtype.NamedType("digestAlgorithms", DigestAlgorithmIdentifiers()),
    namedtype.NamedType("encapContentInfo", EncapsulatedContentInfo()),
    namedtype.OptionalNamedType(
        "certificates",
        CertificateSet().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "crls",
        RevocationInfoChoices().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType("signerInfos", SignerInfos()),
)

id_signingTime = _buildOid(1, 2, 840, 113549, 1, 9, 5)


class SigningTime(Time):
    pass


id_ct_authData = _buildOid(1, 2, 840, 113549, 1, 9, 16, 1, 2)


# CMS Content Type Map

_cmsContentTypesMapUpdate = {
    id_ct_contentInfo: ContentInfo(),
    id_data: univ.OctetString(),
    id_signedData: SignedData(),
    id_envelopedData: EnvelopedData(),
    id_digestedData: DigestedData(),
    id_encryptedData: EncryptedData(),
    id_ct_authData: AuthenticatedData(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)


# CMS Attribute Map

_cmsAttributesMapUpdate = {
    id_contentType: ContentType(),
    id_messageDigest: MessageDigest(),
    id_signingTime: SigningTime(),
    id_countersignature: Countersignature(),
}

cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Object Identifiers for Test Certificate Policies
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7229.txt
#

from pyasn1.type import univ

id_pkix = univ.ObjectIdentifier("1.3.6.1.5.5.7")

id_TEST = id_pkix + (13,)

id_TEST_certPolicyOne = id_TEST + (1,)
id_TEST_certPolicyTwo = id_TEST + (2,)
id_TEST_certPolicyThree = id_TEST + (3,)
id_TEST_certPolicyFour = id_TEST + (4,)
id_TEST_certPolicyFive = id_TEST + (5,)
id_TEST_certPolicySix = id_TEST + (6,)
id_TEST_certPolicySeven = id_TEST + (7,)
id_TEST_certPolicyEight = id_TEST + (8,)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc5480, rfc5915


class MUDCertTestCase(unittest.TestCase):
    private_key_pem_text = """\
MIGkAgEBBDDLjzGbbLrR3T13lrrVum7WC/4Ua4Femc1RhhNVe1Q5XsArQ33kn9kx
3lOUfOcG+qagBwYFK4EEACKhZANiAAT4zZ8HL+xEDpXWkoWp5xFMTz4u4Ae1nF6z
XCYlmsEGD5vPu5hl9hDEjd1UHRgJIPoy3fJcWWeZ8FHCirICtuMgFisNscG/aTwK
yDYOFDuqz/C2jyEwqgWCRyxyohuJXtk=
"""

    def setUp(self):
        self.asn1Spec = rfc5915.ECPrivateKey()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.private_key_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5480.secp384r1, asn1Object["parameters"]["namedCurve"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Multicast Email (MULE) over Allied Communications Publication 142
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8494.txt

from pyasn1.type import namedtype, namedval, tag, univ

id_mmhs_CDT = univ.ObjectIdentifier("1.3.26.0.4406.0.4.2")


class AlgorithmID_ShortForm(univ.Integer):
    pass


AlgorithmID_ShortForm.namedValues = namedval.NamedValues(("zlibCompress", 0))


class ContentType_ShortForm(univ.Integer):
    pass


ContentType_ShortForm.namedValues = namedval.NamedValues(
    ("unidentified", 0), ("external", 1), ("p1", 2), ("p3", 3), ("p7", 4), ("mule", 25)
)


class CompressedContentInfo(univ.Sequence):
    pass


CompressedContentInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "unnamed",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "contentType-ShortForm",
                    ContentType_ShortForm().subtype(
                        explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
                    ),
                ),
                namedtype.NamedType(
                    "contentType-OID",
                    univ.ObjectIdentifier().subtype(
                        explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
                    ),
                ),
            )
        ),
    ),
    namedtype.NamedType(
        "compressedContent",
        univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class CompressionAlgorithmIdentifier(univ.Choice):
    pass


CompressionAlgorithmIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "algorithmID-ShortForm",
        AlgorithmID_ShortForm().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "algorithmID-OID",
        univ.ObjectIdentifier().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class CompressedData(univ.Sequence):
    pass


CompressedData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("compressionAlgorithm", CompressionAlgorithmIdentifier()),
    namedtype.NamedType("compressedContentInfo", CompressedContentInfo()),
)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Multiple Signatures in Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5752.txt
# https://www.rfc-editor.org/errata/eid4444
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5035, rfc5652


class SignAttrsHash(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algID", rfc5652.DigestAlgorithmIdentifier()),
        namedtype.NamedType("hash", univ.OctetString()),
    )


class MultipleSignatures(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("bodyHashAlg", rfc5652.DigestAlgorithmIdentifier()),
        namedtype.NamedType("signAlg", rfc5652.SignatureAlgorithmIdentifier()),
        namedtype.NamedType("signAttrsHash", SignAttrsHash()),
        namedtype.OptionalNamedType("cert", rfc5035.ESSCertIDv2()),
    )


id_aa_multipleSignatures = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.51")


# Map of Attribute Type OIDs to Attributes added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_multipleSignatures: MultipleSignatures(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# X.509 Certificate Extension for Hash Of Root Key
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8649.txt
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5280

id_ce_hashOfRootKey = univ.ObjectIdentifier("1.3.6.1.4.1.51483.2.1")


class HashedRootKey(univ.Sequence):
    pass


HashedRootKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hashAlg", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("hashValue", univ.OctetString()),
)


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_ce_hashOfRootKey: HashedRootKey(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)
